void RunFmsSeePi0(int nevents = 3e3, const char* file = "9065073.list")
{
  // Load shared libraries
  gROOT->Macro("loadMuDst.C");
  gROOT->Macro("LoadLogger.C");
  gSystem->Load("StTpcDb");
  gSystem->Load("StDetectorDbMaker");
  gSystem->Load("StDbUtilities");
  gSystem->Load("StDaqLib");
  gSystem->Load("StEmcRawMaker");
  gSystem->Load("StEmcADCtoEMaker");
  gSystem->Load("StPreEclMaker");
  gSystem->Load("StEpcMaker");
  gSystem->Load("StDbBroker");
  gSystem->Load("St_db_Maker");

  // Load your shared libraries here
  gSystem->Load("StFmsAnalysisMaker");

  // Create chain
  StChain* chain = new StChain;

  // Instantiate MuDst maker
  StMuDstMaker* muDstMaker = new StMuDstMaker(0,0,"",file,".",1000,"MuDst");
  StMuDbReader* muDB = StMuDbReader::instance();

  // Database
  //St_db_Maker* starDb = new St_db_Maker("StarDb", "MySQL:StarDb");
  
  // Instantiate your makers here
  StFmsAnalysisMaker* fms = new StFmsAnalysisMaker;
  fms->SetFile("fms.root");
  TCanvas* c1=new TCanvas("c1","c1",600,600);
  c1->Divide(2,1);
  c1->GetPad(2)->SetLogy();
  // Initialize chain
  chain->Init();
  StFmsAnalysisMaker* fmsMaker=chain->GetMaker("StFmsAnalysisMaker");
  fmsMaker->SetPrintLevel(0);

  TH2F* FmsH[4];
  TH1F* SummedEnergy=new TH1F("SummedEnergy","SummedFMSEnergy",120,0,120);
  SummedEnergy->GetXaxis()->SetTitle("Energy in 4 FMS detectors (GeV)");
  // Event loop
  cout<<"Prepare to process "<<nevents<<" events.\n";
  for (int iEvent = 1; iEvent <= nevents; ++iEvent) 
    {
      if(iEvent%100==1)cout<<"iEvent="<<iEvent<<"\n";
      chain->Clear();
      int status = chain->Make(iEvent);
      if (status % 10 == kStEOF || status % 10 == kStFatal) break;
      /*
      TH2F NorthLarge(*(fmsMaker->GetEnergy(0)));
      NorthLarge.SetName("NorthLargeFMS");
      c1->cd(1);
      NorthLarge.Draw("zcol");
      TH2F SouthLarge(*(fmsMaker->GetEnergy(1)));
      SouthLarge.SetName("SouthLargeFMS");
      c1->cd(2);
      SouthLarge.Draw("zcol");
      TH2F NorthSmall(*(fmsMaker->GetEnergy(2)));
      NorthSmall.SetName("NorthSmallFMS");
      c1->cd(3);
      NorthSmall.Draw("zcol");
      TH2F SouthSmall(*(fmsMaker->GetEnergy(3)));
      SouthSmall.SetName("SouthSmallFMS");
      c1->cd(4);
      SouthSmall.Draw("zcol");
      c1->Update();
      */
      Float_t eventEnergy=0;
      for(Int_t k=0;k<4;k++)eventEnergy+=fmsMaker->GetEnergy(k)->Sum();
      SummedEnergy->Fill(eventEnergy);

      if(iEvent%1000==1)
	{
	  c1->cd(1);
	  TH2F* hmass=fmsMaker->get_MvsE();
	  hmass->GetXaxis()->SetTitle("2 Photon Mass (GeV)");
	  hmass->ProjectionX("EnergyGT10",11,100)->Draw();
	  c1->cd(2);
	  SummedEnergy->Draw();
	  cout<<"Update****\n";
	  c1->Update();
	}; 
    }
}
  
