// ROOT
#include "TFile.h"
#include "TH1.h"
#include "TH2.h"
#include "TMatrix.h"
// STAR
#include "StEventTypes.h"
#include "StMuDSTMaker/COMMON/StMuTypes.hh"

// PSU-FMS package
#include "FilesSet.h"
#include "Geom.h"
#include "Qt.h"
#include "CalibStr.h"
#include "Yiqun.h"
// Local
#include "StFmsAnalysisMaker.h"
#include "StFmsTriggerDetectorAccess.h"


//per Pibero
#include <iostream>
#include <fstream>
using namespace std;

ClassImp(StFmsAnalysisMaker);

void StFmsAnalysisMaker::Clear(Option_t* option)
{
  for(int k=0;k<4;k++)*(Energy[k])=0;
  StMaker::Clear(option);
}

int StFmsAnalysisMaker::Init()
{
  // Create a new ROOT file. Note, any existing file
  // will be overwritten.

  mFile = TFile::Open(mFileName,"recreate");

  // Make sure the file was created successfully by
  // checking that mFile is non-zero.

  assert(mFile);
  FitTower::we.myRand=NULL; //ugly global stuff
  // Read in pedestal, gains, calibration, etc. files
  //  via PSU package
 mFiles = new FilesSet("/star/u/heppel/BatchDir/rootfms",
                        "lb_trg2008/fms/fmsped_2008.txt",
                        "lb_trg2008/fms/fmsgain.20080526.txt",
                        "lb_trg2008/fms/fpdcorr_itr10_20080828.txt",
                        "fpd08/fill.txt",
                        "Fake",
                        "fpd08/spinpat",
                        "lb_trg2008/fms/geom_2008pp_0227.txt",
                        "lb_trg2008/fms/qtmap.txt",
                        "lb_trg2008/fms/qtmap2pp_set121.txt");


  mGeom = new Geom(mFiles);
  mQt=new Qt(mFiles);
  mQt->EnableqtHist();
  MvsE=new TH2F("MvsE","MvsE",200,0.,2.,100,0.,100.);
  currentRunNumber= 9067059;  // Some number from run 8
  
  Rped=new CalibStr(currentRunNumber,mFiles->p_fpdped()->Path());
  Rgain=new CalibStr(currentRunNumber,mFiles->p_fpdgain()->Path());
  Rgaincorr=new CalibStr(currentRunNumber,mFiles->p_fpdgaincorr()->Path());
  for(int k=0;k<4;k++)
    {
      Energy[k]=new TMatrix(*(Rgaincorr->tm(2,k+1)));
      *(Energy[k])=0.;
    };

  PrintLevel=1;
  // Create some histograms


  return StMaker::Init();
}

int StFmsAnalysisMaker::Make()
{
  // Get MuDst dataset. If the call to GetDataSet() returns 0,
  // then no MuDst was found. At which point, issue a warning
  // and exit Make().
  
  if (!GetDataSet("MuDst")) 
    {
      LOG_WARN << "No MuDst" << endm;
      return kStWarn;
    }
  
  UInt_t nhits=((StFmsTriggerDetectorAccess) 
		(StMuDst::event()->fmsTriggerDetector())).GetmNumQTdata();
  
  UInt_t* QTdata=((StFmsTriggerDetectorAccess) 
                  (StMuDst::event()->fmsTriggerDetector())).GetmQTdata();
  if(PrintLevel)  cout<<nhits<<" hits \n";
  mQt->decodeQT(nhits,QTdata,0);
  TMatrix adcFmsNLarge=*(mQt->tm(0));
  TMatrix adcFmsSLarge=*(mQt->tm(1));
  TMatrix adcFmsNSmall=*(mQt->tm(2));
  TMatrix adcFmsSSmall=*(mQt->tm(3));
  TMatrix* adc[4];
  Int_t Nphotons[4];
  Double_t M12[4];
  adc[0]=&adcFmsNLarge;
  adc[1]=&adcFmsSLarge;
  adc[2]=&adcFmsNSmall;
  adc[3]=&adcFmsSSmall;
  for(Int_t j=0;j<4;j++)
    {
      *Energy[j]=0;
    }
  Yiqun* p_rec[4];
  for(Int_t j=0;j<4;j++)
    {
      *(Energy[j])=*(adc[j])-*(Rped->tm(2,j+1));
      
      for(Int_t row=0;row<adc[j]->GetNrows();row++)
        {
          for(Int_t col=0;col<adc[j]->GetNcols();col++)
            {
              Float_t e1=(*(Energy[j]))[row][col];
              Float_t g1=(*(Rgain->tm(2,j+1)))[row][col];
              Float_t g2=(*(Rgaincorr->tm(2,j+1)))[row][col];
              (*(Energy[j]))[row][col]=e1*g1*g2;
            };
        };
      p_rec[j]=new Yiqun(Energy[j],mGeom,Rgain,Rgaincorr,2,j+1);
      Nphotons[j]=p_rec[j]->NPh;
      M12[j]=0.;
      if(Nphotons[j]>1)
	{
	  M12[j]=(p_rec[j]->mom(0)+p_rec[j]->mom(1)).Mag();
	  Float_t E12=(p_rec[j]->mom(0)+p_rec[j]->mom(1)).E();
	  if(Nphotons[j]==2)MvsE->Fill(M12[j],E12);
	};

    };
  for(int j=0;j<4;j++){
    delete p_rec[j];


  };
  
  if(PrintLevel)
    {
      cout<<"Energy(NLarge,SLarge,NSmall,SSmall)=(\n";
      cout<<Energy[0]->Sum()<<" GeV : Nph="<<Nphotons[0]<<":M12="<<M12[0]<<" GeV,\n";
      cout<<Energy[1]->Sum()<<" GeV : Nph="<<Nphotons[1]<<":M12="<<M12[1]<<" GeV,\n";
      cout<<Energy[2]->Sum()<<" GeV : Nph="<<Nphotons[2]<<":M12="<<M12[2]<<" GeV,\n";
      cout<<Energy[3]->Sum()<<" GeV): Nph="<<Nphotons[3]<<":M12="<<M12[3]<<" GeV)\n";
      cout<<"Event done\n";
    };
  // The event was processed successfully.
  
  return kStOk;
}

int StFmsAnalysisMaker::Finish()
{
  // Write histograms to ROOT file and close it.

  mFile->Write();
  mFile->Close();

  return kStOk;
}
